<?php

ini_set('memory_limit', '1G');
include('CNC_Equals_CONSTS.php');

interface RequestInterface {
    public function send( $type );
    public function get();
    public function post();
    public function put();
    public function delete();

    // GETTERS
    public function getUrl();
    public function getParams();
    public function getHeader();

    // SETTERS
    public function setUrl( $url );
    public function setParams( $params );
    public function setHeader( $header );
}

abstract class RequestMethods implements RequestInterface {
    public function send( $type ){
        $curl = curl_init();

        curl_setopt_array($curl, [
            CURLOPT_URL => $this->getUrl(),
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => $type,
            CURLOPT_POSTFIELDS => json_encode($this->getParams()),
            CURLOPT_HTTPHEADER => $this->getHeader()
        ]);
        $response = curl_exec($curl);
        $err = curl_error($curl);
        
        curl_close($curl);
        if($err) {
            var_dump($err);
        }

        return $err ? ("cURL Error #:" . $err) : json_decode($response);
    }

    public function get(){
        return $this->send("GET");
    }
    
    public function post(){
        return $this->send("POST");
    }
    
    public function put(){
        return $this->send("PUT");
    }
    public function delete(){
        return $this->send("DELETE");
    }
}

class Request extends RequestMethods {

    private $url = "";
    private $params = [];
    private $header = [];
    
    public function __construct( $url, $params, $header = [ "Content-Type: application/json" ] ){
        $this->url = $url;
        $this->params = $params;
        $this->header = $header;
    }

    public function addHeaderParam( $param ) {
        $this->header[] = $param;
        return $this;
    }

    public function addHeaderParams( $params ){
        $this->header = array_merge($this->header, $params);
        return $this;
    }

    public function addBasicAuth( $token ){
        $this->header[] = "Authorization: Basic {$token}";
        return $this;
    }

    public function getUrl()    { return $this->url;    }
    public function getParams() { return $this->params; }
    public function getHeader() { return $this->header; }

    public function setUrl( $url )       { $this->url    = $url;    }
    public function setParams( $params ) { $this->params = $params; }
    public function setHeader( $header ) { $this->header = $header; }

}

interface ConciliationInterface {
    public static function getSales( $filter );
    public static function getSalesFromPeriod( $startDate,  $endDate);

    public static function getPayments( $filter );
    public static function getPaymentsFromPeriod( $startDate, $endDate );

    public static function getFormattedDateParams( $startDate, $endDate );
}

abstract class ConciliationApi implements ConciliationInterface {

    public static function getPaymentsFromPeriod( $startDate, $endDate ){
        return $this->getPayments(
            $this->getFormattedDateParams($startDate, $endDate)
        );
    }

    public static function getSalesFromPeriod( $startDate, $endDate ) {
        return $this->getSales(
            $this->getFormattedDateParams($startDate, $endDate)
        );
    }
}

class EqualsApi extends ConciliationApi {

    private static $token = '';

    public static function setToken($token) {
        self::$token = $token;
    }

    public static function getToken() {
        return EqualsApi::$token;
    }

    public static function getFormattedDateParams( $startDate, $endDate ) {
        return [
            "dataInicial" => $startDate,
            "dataFinal"   => $endDate
        ];
    }

    public static function getStandardRequest($url, $params){
        $request = new Request($url, $params);
        $request->addBasicAuth(EqualsApi::getToken());
        return $request;
    }

    public static function getSales( $filter = [] ){
        $saleUrl = API_URL . '/venda';
        $request = EqualsApi::getStandardRequest($saleUrl, $filter);
        return $request->post();
    }

    public static function getSalesFromPeriod( $startDate, $endDate ){
        $filterDate = EqualsApi::getFormattedDateParams($startDate, $endDate);
        return EqualsApi::getSales($filterDate);
    }

    public static function getPayments( $filter = [] ){
        $paymentUrl = API_URL . '/vendaerppagamento';
        $request = EqualsApi::getStandardRequest($paymentUrl, $filter);
        return $request->post();
    }

    public static function getPaymentsFromPeriod( $startDate, $endDate ){
        $filterDate = EqualsApi::getFormattedDateParams($startDate, $endDate);
        return EqualsApi::getPayments($filterDate);
    }

}